{% if graph_data %}
<pre class="mermaid">
{{ graph_data }}
</pre>
{% else %}
<p style="color:var(--text-muted); padding:2rem; text-align:center;">No tasks yet. Create some tasks to see the graph.</p>
{% endif %}
