{% extends "base.html" %}
{% block head %}
<script src="https://cdn.jsdelivr.net/npm/mermaid@11/dist/mermaid.min.js"></script>
{% endblock %}
{% block content %}
<h1>Dependency Graph — {{ project_name }}</h1>
<div class="graph-container"
     id="graph-view"
     hx-get="/dashboard/api/graph"
     hx-trigger="load, sse:task_update"
     hx-swap="innerHTML">
</div>
{% endblock %}
