<div class="project-grid">
    {% for p in projects %}
    <div class="project-card">
        <h3>{{ p.name }}</h3>
        {% if p.description %}<p style="font-size:0.85rem; color:var(--text-muted);">{{ p.description }}</p>{% endif %}
        <div class="progress-bar">
            <div class="progress-fill" style="width: {{ p.pct }}%"></div>
        </div>
        <div style="font-size:0.8rem; color:var(--text-muted); display:flex; justify-content:space-between;">
            <span>{{ p.done }}/{{ p.total }} done</span>
            <span>{{ p.pct }}%</span>
        </div>
        <div style="font-size:0.8rem; margin-top:0.5rem;">
            <code>{{ p.id }}</code>
        </div>
    </div>
    {% endfor %}
    {% if not projects %}
    <p style="color:var(--text-muted);">No projects found.</p>
    {% endif %}
</div>
