{% extends "base.html" %}
{% block content %}
<h1>Task Board — {{ project_name }}</h1>

<div class="stats">
    <div class="stat"><div class="value">{{ stats.total }}</div><div class="label">Total</div></div>
    <div class="stat"><div class="value">{{ stats.pending }}</div><div class="label">Pending</div></div>
    <div class="stat"><div class="value">{{ stats.claimed }}</div><div class="label">Claimed</div></div>
    <div class="stat"><div class="value">{{ stats.done }}</div><div class="label">Done</div></div>
    <div class="stat"><div class="value">{{ stats.failed }}</div><div class="label">Failed</div></div>
    <div class="stat"><div class="value">{{ stats.blocked }}</div><div class="label">Blocked</div></div>
</div>

<div id="task-board"
     hx-get="/dashboard/api/tasks"
     hx-trigger="load, sse:task_update"
     hx-swap="innerHTML">
</div>

<div id="task-detail"></div>
{% endblock %}
