{% extends "base.html" %}
{% block content %}
<h1>Projects</h1>
<div id="project-list"
     hx-get="/dashboard/api/projects"
     hx-trigger="load, sse:project_update"
     hx-swap="innerHTML">
</div>
{% endblock %}
